@@ -39,14 +39,17 @@
  *  - Backport hw cursor acceleration to Basilisk II?
  *  - Factor out code
  */
+
 #include "sysdeps.h"
 
-#ifdef ENABLE_SDL1
 #include <SDL.h>
+#if (SDL_COMPILEDVERSION < SDL_VERSIONNUM(2, 0, 0))
+
 #include <SDL_mutex.h>
 #include <SDL_thread.h>
 #include <errno.h>
 #include <vector>
+#include <string>
 
 #ifdef WIN32
 #include <malloc.h> /* alloca() */
@@ -86,15 +89,12 @@ static int display_type = DISPLAY_WINDOW;			// See enum above
 #endif
 
 // Constants
-#ifdef WIN32
+#if defined(WIN32) || __MACOSX__
 const char KEYCODE_FILE_NAME[] = "BasiliskII_keycodes";
 #else
 const char KEYCODE_FILE_NAME[] = DATADIR "/keycodes";
 #endif
 
-// Mac Screen Width and Height
-uint32 MacScreenWidth;
-uint32 MacScreenHeight;
 
 // Global variables
 static uint32 frame_skip;							// Prefs items
@@ -119,7 +119,9 @@ static bool use_vosf = false;						// Flag: VOSF enabled
 static const bool use_vosf = false;					// VOSF not possible
 #endif
 
-static bool ctrl_down = false;						// Flag: Ctrl key pressed
+static bool ctrl_down = false;						// Flag: Ctrl key pressed (for use with hotkeys)
+static bool opt_down = false;						// Flag: Opt/Alt key pressed (for use with hotkeys)
+static bool cmd_down = false;						// Flag: Cmd/Super/Win key pressed (for use with hotkeys)
 static bool caps_on = false;						// Flag: Caps Lock on
 static bool quit_full_screen = false;				// Flag: DGA close requested from redraw thread
 static bool emerg_quit = false;						// Flag: Ctrl-Esc pressed, emergency quit requested from MacOS thread
@@ -156,6 +158,11 @@ static SDL_mutex *frame_buffer_lock = NULL;
 #define LOCK_FRAME_BUFFER SDL_LockMutex(frame_buffer_lock)
 #define UNLOCK_FRAME_BUFFER SDL_UnlockMutex(frame_buffer_lock)
 
+// Previously set gamma tables
+static uint16 last_gamma_red[256];
+static uint16 last_gamma_green[256];
+static uint16 last_gamma_blue[256];
+
 // Video refresh function
 static void VideoRefreshInit(void);
 static void (*video_refresh)(void);
@@ -200,7 +207,18 @@ extern void SysMountFirstFloppy(void);
 
 static void *vm_acquire_framebuffer(uint32 size)
 {
-	void *fb = vm_acquire(size, VM_MAP_DEFAULT | VM_MAP_32BIT);
+	// always try to reallocate framebuffer at the same address
+	static void *fb = VM_MAP_FAILED;
+	if (fb != VM_MAP_FAILED) {
+		if (vm_acquire_fixed(fb, size) < 0) {
+#ifndef SHEEPSHAVER
+			printf("FATAL: Could not reallocate framebuffer at previous address\n");
+#endif
+			fb = VM_MAP_FAILED;
+		}
+	}
+	if (fb == VM_MAP_FAILED)
+		fb = vm_acquire(size, VM_MAP_DEFAULT | VM_MAP_32BIT);
 	return fb;
 }
 
@@ -350,6 +368,7 @@ public:
 
 	virtual void switch_to_current_mode(void);
 	virtual void set_palette(uint8 *pal, int num);
+	virtual void set_gamma(uint8 *gamma, int num);
 
 	bool video_open(void);
 	void video_close(void);
@@ -450,12 +469,6 @@ static inline int sdl_display_height(void)
 // Check whether specified mode is available
 static bool has_mode(int type, int width, int height, int depth)
 {
-#ifdef SHEEPSHAVER
-	// Filter out Classic resolutions
-	if (width == 512 && height == 384)
-		return false;
-#endif
-
 	// Filter out out-of-bounds resolutions
 	if (width > sdl_display_width() || height > sdl_display_height())
 		return false;
@@ -497,11 +510,7 @@ static void set_mac_frame_buffer(SDL_monitor_desc &monitor, int depth)
 	else if (depth == VIDEO_DEPTH_32BIT)
 		layout = (screen_depth == 24) ? FLAYOUT_HOST_888 : FLAYOUT_DIRECT;
 	MacFrameLayout = layout;
-
-	if (TwentyFourBitAddressing)
-		monitor.set_mac_frame_base(MacFrameBaseMac24Bit);
-	else
-		monitor.set_mac_frame_base(MacFrameBaseMac);
+	monitor.set_mac_frame_base(MacFrameBaseMac);
 
 	// Set variables used by UAE memory banking
 	const VIDEO_MODE &mode = monitor.get_current_mode();
@@ -515,12 +524,26 @@ static void set_mac_frame_buffer(SDL_monitor_desc &monitor, int depth)
 }
 
 // Set window name and class
-static void set_window_name(int name)
+static void set_window_name(bool mouse_grabbed)
 {
+	const char *title = PrefsFindString("title");
+	std::string s = title ? title : GetString(STR_WINDOW_TITLE);
+	int grabbed = 0;
+	if (mouse_grabbed)
+	{
+        s += GetString(STR_WINDOW_TITLE_GRABBED_PRE);
+		int hotkey = PrefsFindInt32("hotkey");
+		hotkey = hotkey ? hotkey : 1;
+		if (hotkey & 1) s += GetString(STR_WINDOW_TITLE_GRABBED1);
+        if (hotkey & 2) s += GetString(STR_WINDOW_TITLE_GRABBED2);
+        if (hotkey & 4) s += GetString(STR_WINDOW_TITLE_GRABBED4);
+        s += GetString(STR_WINDOW_TITLE_GRABBED_POST);
+	}
 	const SDL_VideoInfo *vi = SDL_GetVideoInfo();
-	if (vi && vi->wm_available) {
-		const char *str = GetString(name);
-		SDL_WM_SetCaption(str, str);
+	if (vi && vi->wm_available)
+	{
+		//The icon name should stay the same
+		SDL_WM_SetCaption(s.c_str(), GetString(STR_WINDOW_TITLE));
 	}
 }
 
@@ -651,10 +674,6 @@ void driver_base::set_video_mode(int flags)
 #ifdef ENABLE_VOSF
 	the_host_buffer = (uint8 *)s->pixels;
 #endif
-	// set Mac screen global variabls
-	MacScreenWidth = VIDEO_MODE_X;
-	MacScreenHeight = VIDEO_MODE_Y;
-	D(bug("Set Mac Screen Width: %d, Mac Screen Height: %d\n", MacScreenWidth, MacScreenHeight));
 }
 
 void driver_base::init()
@@ -667,10 +686,6 @@ void driver_base::init()
 	// Allocate memory for frame buffer (SIZE is extended to page-boundary)
 	the_buffer_size = page_extend((aligned_height + 2) * s->pitch);
 	the_buffer = (uint8 *)vm_acquire_framebuffer(the_buffer_size);
-	if (VM_MAP_FAILED == the_buffer) {
-		perror("Failed to allocate frame buffer for guest OS.");
-		abort();
-	}
 	the_buffer_copy = (uint8 *)malloc(the_buffer_size);
 	D(bug("the_buffer = %p, the_buffer_copy = %p, the_host_buffer = %p\n", the_buffer, the_buffer_copy, the_host_buffer));
 
@@ -695,10 +710,6 @@ void driver_base::init()
 		the_buffer_size = (aligned_height + 2) * s->pitch;
 		the_buffer_copy = (uint8 *)calloc(1, the_buffer_size);
 		the_buffer = (uint8 *)vm_acquire_framebuffer(the_buffer_size);
-		if (VM_MAP_FAILED == the_buffer) {
-			perror("Failed to allocate frame buffer for guest OS.");
-			abort();
-		}
 		D(bug("the_buffer = %p, the_buffer_copy = %p\n", the_buffer, the_buffer_copy));
 	}
 
@@ -743,7 +754,7 @@ void driver_base::adapt_to_video_mode() {
 	SDL_ShowCursor(hardware_cursor);
 
 	// Set window name/class
-	set_window_name(STR_WINDOW_TITLE);
+	set_window_name(false);
 
 	// Everything went well
 	init_ok = true;
@@ -1005,13 +1016,16 @@ bool VideoInit(bool classic)
 	// Get screen mode from preferences
 	migrate_screen_prefs();
 	const char *mode_str = NULL;
-	mode_str = PrefsFindString("screen");
+	if (classic_mode)
+		mode_str = "win/512/342";
+	else
+		mode_str = PrefsFindString("screen");
 
 	// Determine display type and default dimensions
 	int default_width, default_height;
 	if (classic) {
 		default_width = 512;
-		default_height = 342;
+		default_height = 384;
 	}
 	else {
 		default_width = 640;
@@ -1033,11 +1047,6 @@ bool VideoInit(bool classic)
 	else if (default_height > sdl_display_height())
 		default_height = sdl_display_height();
 
-	// for classic Mac, make sure the display width is divisible by 8
-	if (classic) {
-		default_width = (default_width / 8) * 8;
-	}
-
 	// Mac screen depth follows X depth
 	screen_depth = SDL_GetVideoInfo()->vfmt->BitsPerPixel;
 	int default_depth;
@@ -1062,6 +1071,19 @@ bool VideoInit(bool classic)
 		int h;
 		int resolution_id;
 	}
+#ifdef SHEEPSHAVER
+	// Omit Classic resolutions
+	video_modes[] = {
+		{   -1,   -1, 0x80 },
+		{  640,  480, 0x81 },
+		{  800,  600, 0x82 },
+		{ 1024,  768, 0x83 },
+		{ 1152,  870, 0x84 },
+		{ 1280, 1024, 0x85 },
+		{ 1600, 1200, 0x86 },
+		{ 0, }
+	};
+#else
 	video_modes[] = {
 		{   -1,   -1, 0x80 },
 		{  512,  384, 0x80 },
@@ -1073,13 +1095,14 @@ bool VideoInit(bool classic)
 		{ 1600, 1200, 0x86 },
 		{ 0, }
 	};
+#endif
 	video_modes[0].w = default_width;
 	video_modes[0].h = default_height;
 
 	// Construct list of supported modes
 	if (display_type == DISPLAY_WINDOW) {
 		if (classic)
-			add_mode(display_type, default_width, default_height, 0x80, default_width/8, VIDEO_DEPTH_1BIT);
+			add_mode(display_type, 512, 342, 0x80, 64, VIDEO_DEPTH_1BIT);
 		else {
 			for (int i = 0; video_modes[i].w != 0; i++) {
 				const int w = video_modes[i].w;
@@ -1299,8 +1322,8 @@ void VideoVBL(void)
 	// Setting the window name must happen on the main thread, else it doesn't work on
 	// some platforms - e.g. macOS Sierra.
 	if (mouse_grabbed_window_name_status != mouse_grabbed) {
-		set_window_name(mouse_grabbed ? STR_WINDOW_TITLE_GRABBED : STR_WINDOW_TITLE);
-		mouse_grabbed_window_name_status = mouse_grabbed;
+	    set_window_name(mouse_grabbed);
+	    mouse_grabbed_window_name_status = mouse_grabbed;
 	}
 
 	// Temporarily give up frame buffer lock (this is the point where
@@ -1328,7 +1351,7 @@ void VideoInterrupt(void)
 	// Setting the window name must happen on the main thread, else it doesn't work on
 	// some platforms - e.g. macOS Sierra.
 	if (mouse_grabbed_window_name_status != mouse_grabbed) {
-		set_window_name(mouse_grabbed ? STR_WINDOW_TITLE_GRABBED : STR_WINDOW_TITLE);
+		set_window_name(mouse_grabbed);
 		mouse_grabbed_window_name_status = mouse_grabbed;
 	}
 
@@ -1363,9 +1386,51 @@ void SDL_monitor_desc::set_palette(uint8 *pal, int num_in)
 {
 	const VIDEO_MODE &mode = get_current_mode();
 
-	// FIXME: how can we handle the gamma ramp?
-	if ((int)VIDEO_MODE_DEPTH > VIDEO_DEPTH_8BIT)
+	if ((int)VIDEO_MODE_DEPTH > VIDEO_DEPTH_8BIT) {
+		// handle the gamma ramp
+
+		if (pal[0] == 127 && pal[num_in*3-1] == 127) // solid grey
+			return; // ignore
+
+		uint16 red[256];
+		uint16 green[256];
+		uint16 blue[256];
+		
+		int repeats = 256 / num_in;
+				
+		for (int i = 0; i < num_in; i++) {
+			for (int j = 0; j < repeats; j++) {
+				red[i*repeats + j] = pal[i*3 + 0] << 8;
+				green[i*repeats + j] = pal[i*3 + 1] << 8;
+				blue[i*repeats + j] = pal[i*3 + 2] << 8;
+			}
+		}
+
+		// fill remaining entries (if any) with last value
+		for (int i = num_in * repeats; i < 256; i++) {
+			red[i] = pal[(num_in - 1) * 3] << 8;
+			green[i] = pal[(num_in - 1) * 3 + 1] << 8;
+			blue[i] = pal[(num_in - 1) * 3 + 2] << 8;
+		}
+		
+		bool changed = (memcmp(red, last_gamma_red, 512) != 0 ||
+		                memcmp(green, last_gamma_green, 512) != 0 ||
+		                memcmp(blue, last_gamma_blue, 512) != 0);
+		
+		if (changed) {
+			int result = SDL_SetGammaRamp(red, green, blue);
+
+			if (result < 0) {
+				fprintf(stderr, "SDL_SetGammaRamp returned %d, SDL error: %s\n", result, SDL_GetError());
+			}
+			
+			memcpy(last_gamma_red, red, 512);
+			memcpy(last_gamma_green, green, 512);
+			memcpy(last_gamma_blue, blue, 512);
+		}
+
 		return;
+	}
 
 	LOCK_PALETTE;
 
@@ -1405,6 +1470,10 @@ void SDL_monitor_desc::set_palette(uint8 *pal, int num_in)
 	UNLOCK_PALETTE;
 }
 
+void SDL_monitor_desc::set_gamma(uint8 *gamma, int num_in)
+{
+	// Not implemented
+}
 
 /*
  *  Switch video mode
@@ -1564,11 +1633,29 @@ static bool is_modifier_key(SDL_KeyboardEvent const & e)
 	return false;
 }
 
-static bool is_ctrl_down(SDL_keysym const & ks)
+static bool is_hotkey_down(SDL_keysym const & ks)
 {
-	return ctrl_down || (ks.mod & KMOD_CTRL);
+	int hotkey = PrefsFindInt32("hotkey");
+	if (!hotkey) hotkey = 1;
+	return (ctrl_down || (ks.mod & KMOD_CTRL) || !(hotkey & 1)) &&
+			(opt_down || (ks.mod & KMOD_ALT) || !(hotkey & 2)) &&
+			(cmd_down || (ks.mod & KMOD_META) || !(hotkey & 4));
 }
 
+static int modify_opt_cmd(int code) {
+	static bool f, c;
+	if (!f) {
+		f = true;
+		c = PrefsFindBool("swap_opt_cmd");
+	}
+	if (c) {
+		switch (code) {
+			case 0x37: return 0x3a;
+			case 0x3a: return 0x37;
+		}
+	}
+	return code;
+}
 
 /*
  *  Translate key event to Mac keycode, returns -1 if no keycode was found
@@ -1628,8 +1715,8 @@ static int kc_decode(SDL_keysym const & ks, bool key_down)
 	case SDLK_PERIOD: case SDLK_GREATER: return 0x2f;
 	case SDLK_SLASH: case SDLK_QUESTION: return 0x2c;
 
-	case SDLK_TAB: if (is_ctrl_down(ks)) {if (!key_down) drv->suspend(); return -2;} else return 0x30;
-	case SDLK_RETURN: if (is_ctrl_down(ks)) {if (!key_down) toggle_fullscreen = true; return -2;} else return 0x24;
+	case SDLK_TAB: if (is_hotkey_down(ks)) {if (!key_down) drv->suspend(); return -2;} else return 0x30;
+	case SDLK_RETURN: if (is_hotkey_down(ks)) {if (!key_down) toggle_fullscreen = true; return -2;} else return 0x24;
 	case SDLK_SPACE: return 0x31;
 	case SDLK_BACKSPACE: return 0x33;
 
@@ -1644,19 +1731,9 @@ static int kc_decode(SDL_keysym const & ks, bool key_down)
 	case SDLK_RCTRL: return 0x36;
 	case SDLK_LSHIFT: return 0x38;
 	case SDLK_RSHIFT: return 0x38;
-#if (defined(__APPLE__) && defined(__MACH__))
-	case SDLK_LALT: return 0x3a;
-	case SDLK_RALT: return 0x3a;
-	case SDLK_LMETA: return 0x37;
-	case SDLK_RMETA: return 0x37;
-#else
-	case SDLK_LALT: return 0x37;
-	case SDLK_RALT: return 0x37;
-	case SDLK_LMETA: return 0x3a;
-	case SDLK_RMETA: return 0x3a;
-#endif
-	case SDLK_LSUPER: return 0x3a; // "Windows" key
-	case SDLK_RSUPER: return 0x3a;
+	case SDLK_LALT: case SDLK_RALT: return 0x3a;
+	case SDLK_LMETA: case SDLK_RMETA: return 0x37;
+	case SDLK_LSUPER: case SDLK_RSUPER: return 0x37; // "Windows" key
 	case SDLK_MENU: return 0x32;
 	case SDLK_CAPSLOCK: return 0x39;
 	case SDLK_NUMLOCK: return 0x47;
@@ -1666,13 +1743,13 @@ static int kc_decode(SDL_keysym const & ks, bool key_down)
 	case SDLK_LEFT: return 0x3b;
 	case SDLK_RIGHT: return 0x3c;
 
-	case SDLK_ESCAPE: if (is_ctrl_down(ks)) {if (!key_down) { quit_full_screen = true; emerg_quit = true; } return -2;} else return 0x35;
+	case SDLK_ESCAPE: if (is_hotkey_down(ks)) {if (!key_down) { quit_full_screen = true; emerg_quit = true; } return -2;} else return 0x35;
 
-	case SDLK_F1: if (is_ctrl_down(ks)) {if (!key_down) SysMountFirstFloppy(); return -2;} else return 0x7a;
+	case SDLK_F1: if (is_hotkey_down(ks)) {if (!key_down) SysMountFirstFloppy(); return -2;} else return 0x7a;
 	case SDLK_F2: return 0x78;
 	case SDLK_F3: return 0x63;
 	case SDLK_F4: return 0x76;
-	case SDLK_F5: if (is_ctrl_down(ks)) {if (!key_down) drv->toggle_mouse_grab(); return -2;} else return 0x60;
+	case SDLK_F5: if (is_hotkey_down(ks)) {if (!key_down) drv->toggle_mouse_grab(); return -2;} else return 0x60;
 	case SDLK_F6: return 0x61;
 	case SDLK_F7: return 0x62;
 	case SDLK_F8: return 0x64;
@@ -1795,6 +1872,15 @@ static void handle_events(void)
 					code = event2keycode(event.key, true);
 				if (code >= 0) {
 					if (!emul_suspended) {
+						if (code == 0x36) {
+							ctrl_down = true;
+						} else if (code == 0x3a) {
+							opt_down = true;
+						    code = modify_opt_cmd(code);
+						} else if (code == 0x37) {
+							cmd_down = true;
+						    code = modify_opt_cmd(code);
+						}
 						if (code == 0x39) {	// Caps Lock pressed
 							if (caps_on) {
 								ADBKeyUp(code);
@@ -1805,8 +1891,6 @@ static void handle_events(void)
 							}
 						} else
 							ADBKeyDown(code);
-						if (code == 0x36)
-							ctrl_down = true;
 					} else {
 						if (code == 0x31)
 							drv->resume();	// Space wakes us up
@@ -1822,6 +1906,15 @@ static void handle_events(void)
 				} else
 					code = event2keycode(event.key, false);
 				if (code >= 0) {
+					if (code == 0x36) {
+						ctrl_down = false;
+					} else if (code == 0x3a) {
+						opt_down = false;
+					    code = modify_opt_cmd(code);
+					} else if (code == 0x37) {
+						cmd_down = false;
+					    code = modify_opt_cmd(code);
+					}
 					if (code == 0x39) {	// Caps Lock released
 						if (caps_on) {
 							ADBKeyUp(code);
@@ -1832,8 +1925,6 @@ static void handle_events(void)
 						}
 					} else
 						ADBKeyUp(code);
-					if (code == 0x36)
-						ctrl_down = false;
 				}
 				break;
 			}
@@ -1894,7 +1985,7 @@ static void update_display_static(driver_base *drv)
 
 	// Check for first column from left and first column from right that have changed
 	if (high) {
-		if ((int)VIDEO_MODE_DEPTH < VIDEO_DEPTH_8BIT) {
+		if (VIDEO_MODE_DEPTH < VIDEO_DEPTH_8BIT) {
 			const int src_bytes_per_row = bytes_per_row;
 			const int dst_bytes_per_row = drv->s->pitch;
 			const int pixels_per_byte = VIDEO_MODE_X / src_bytes_per_row;
@@ -2289,4 +2380,12 @@ void video_set_dirty_area(int x, int y, int w, int h)
 	// XXX handle dirty bounding boxes for non-VOSF modes
 }
 #endif
-#endif //ifdef ENABLE_SDL1
+
+#ifdef SHEEPSHAVER
+void video_set_gamma(int n_colors)
+{
+	// Not supported in SDL 1.2
+}
+#endif
+
+#endif	// ends: SDL version check
