@@ -42,13 +42,14 @@
 #include "sysdeps.h"
 
 #include <SDL.h>
-#if SDL_VERSION_ATLEAST(2,0,0)
+#if SDL_VERSION_ATLEAST(2, 0, 0) && !SDL_VERSION_ATLEAST(3, 0, 0)
 
 #include <SDL_mutex.h>
 #include <SDL_thread.h>
 #include <errno.h>
 #include <vector>
 #include <string>
+#include <math.h>
 
 #ifdef __MACOSX__
 #include "utils_macosx.h"
@@ -68,6 +69,7 @@
 #include "video_defs.h"
 #include "video_blit.h"
 #include "vm_alloc.h"
+#include "cdrom.h"
 
 #define DEBUG 0
 #include "debug.h"
@@ -628,7 +630,7 @@ public:
 	~driver_base();
 
 	void init(); // One-time init
-	void set_video_mode(int flags);
+	void set_video_mode(int flags, int pitch);
 	void adapt_to_video_mode();
 
 	void update_palette(void);
@@ -716,7 +718,7 @@ static int get_mag_rate()
 	return m < 1 ? 1 : m > 4 ? 4 : m;
 }
 
-static SDL_Surface * init_sdl_video(int width, int height, int bpp, Uint32 flags)
+static SDL_Surface *init_sdl_video(int width, int height, int depth, Uint32 flags, int pitch)
 {
     if (guest_surface) {
         delete_sdl_video_surfaces();
@@ -733,13 +735,9 @@ static SDL_Surface * init_sdl_video(int width, int height, int bpp, Uint32 flags
 			shutdown_sdl_video();
 			return NULL;
 		}
-#ifdef __MACOSX__
 		window_flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
 		window_width = desktop_mode.w;
 		window_height = desktop_mode.h;
-#else
-		window_flags |= SDL_WINDOW_FULLSCREEN;
-#endif
 	}
 	
 	if (sdl_window) {
@@ -824,7 +822,11 @@ static SDL_Surface * init_sdl_video(int width, int height, int bpp, Uint32 flags
     }
 
 	SDL_assert(sdl_texture == NULL);
-    sdl_texture = SDL_CreateTexture(sdl_renderer, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STREAMING, width, height);
+#ifdef ENABLE_VOSF
+	sdl_texture = SDL_CreateTexture(sdl_renderer, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STREAMING, width, height);
+#else
+	sdl_texture = SDL_CreateTexture(sdl_renderer, SDL_PIXELFORMAT_BGRA8888, SDL_TEXTUREACCESS_STREAMING, width, height);
+#endif
     if (!sdl_texture) {
         shutdown_sdl_video();
         return NULL;
@@ -836,19 +838,32 @@ static SDL_Surface * init_sdl_video(int width, int height, int bpp, Uint32 flags
 
 	SDL_assert(guest_surface == NULL);
 	SDL_assert(host_surface == NULL);
-    switch (bpp) {
-        case 8:
-            guest_surface = SDL_CreateRGBSurface(0, width, height, 8, 0, 0, 0, 0);
-            break;
-		case 16:
-			guest_surface = SDL_CreateRGBSurface(0, width, height, 16, 0x0000F800, 0x000007E0, 0x0000001F, 0x00000000);
+    switch (depth) {
+		case VIDEO_DEPTH_1BIT:
+		case VIDEO_DEPTH_2BIT:
+		case VIDEO_DEPTH_4BIT:
+			guest_surface = SDL_CreateRGBSurface(0, width, height, 8, 0, 0, 0, 0);
 			break;
-        case 32:
-            guest_surface = SDL_CreateRGBSurface(0, width, height, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
-            host_surface = guest_surface;
+		case VIDEO_DEPTH_8BIT:
+#ifdef ENABLE_VOSF
+			guest_surface = SDL_CreateRGBSurface(0, width, height, 8, 0, 0, 0, 0);
+#else
+			guest_surface = SDL_CreateRGBSurfaceFrom(the_buffer, width, height, 8, pitch, 0, 0, 0, 0);
+#endif
+			break;
+		case VIDEO_DEPTH_16BIT:
+			guest_surface = SDL_CreateRGBSurface(0, width, height, 16, 0xf800, 0x07e0, 0x001f, 0);
+			break;
+		case VIDEO_DEPTH_32BIT:
+#ifdef ENABLE_VOSF
+			guest_surface = SDL_CreateRGBSurface(0, width, height, 32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000);
+#else
+			guest_surface = SDL_CreateRGBSurfaceFrom(the_buffer, width, height, 32, pitch, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
+#endif
+			host_surface = guest_surface;
             break;
         default:
-            printf("WARNING: An unsupported bpp of %d was used\n", bpp);
+            printf("WARNING: An unsupported depth of %d was used\n", depth);
             break;
     }
     if (!guest_surface) {
@@ -936,14 +951,22 @@ static int present_sdl_video()
 	UNLOCK_PALETTE; // passed potential deadlock, can unlock palette
 	
     // Update the host OS' texture
-    void * srcPixels = (void *)((uint8_t *)host_surface->pixels +
-        sdl_update_video_rect.y * host_surface->pitch +
-        sdl_update_video_rect.x * host_surface->format->BytesPerPixel);
+	uint8_t *srcPixels = (uint8_t *)host_surface->pixels +
+		sdl_update_video_rect.y * host_surface->pitch +
+		sdl_update_video_rect.x * host_surface->format->BytesPerPixel;
 
-    if (SDL_UpdateTexture(sdl_texture, &sdl_update_video_rect, srcPixels, host_surface->pitch) != 0) {
-        SDL_UnlockMutex(sdl_update_video_mutex);
+	uint8_t *dstPixels;
+	int dstPitch;
+	if (SDL_LockTexture(sdl_texture, &sdl_update_video_rect, (void **)&dstPixels, &dstPitch) < 0) {
+		SDL_UnlockMutex(sdl_update_video_mutex);
 		return -1;
 	}
+	for (int y = 0; y < sdl_update_video_rect.h; y++) {
+		memcpy(dstPixels, srcPixels, sdl_update_video_rect.w << 2);
+		srcPixels += host_surface->pitch;
+		dstPixels += dstPitch;
+	}
+	SDL_UnlockTexture(sdl_texture);
 
     // We are done working with pixels in host_surface.  Reset sdl_update_video_rect, then let
     // other threads modify it, as-needed.
@@ -984,37 +1007,36 @@ void update_sdl_video(SDL_Surface *s, Sint32 x, Sint32 y, Sint32 w, Sint32 h)
 }
 
 #ifdef SHEEPSHAVER
-static void MagBits(Uint8 *dst, Uint8 *src, int mag) {
-	for (int y = 0; y < 16; y++)
-		for (int x = 0; x < 16; x++) {
-			int sa = 16 * y + x;
-			if (!(src[sa >> 3] & 0x80 >> (sa & 7))) continue;
-			for (int dy = 0; dy < mag; dy++)
-				for (int dx = 0; dx < mag; dx++) {
-					int da = 16 * mag * (mag * y + dy) + mag * x + dx;
-					dst[da >> 3] |= 0x80 >> (da & 7);
-				}
+static void MagBits(Uint8 *dst, Uint8 *src, int size) {
+	float s = 16.f / size;
+	for (int y = 0; y < size; y++)
+		for (int x = 0; x < size; x++) {
+			int sa = 16 * int(y * s) + int(x * s);
+			if (src[sa >> 3] & 0x80 >> (sa & 7)) {
+				int da = (size + 7 & ~7) * y + x;
+				dst[da >> 3] |= 0x80 >> (da & 7);
+			}
 		}
 }
 static SDL_Cursor *MagCursor(bool hot) {
 	int w, h;
 	SDL_GetWindowSize(sdl_window, &w, &h);
-	int mag = std::min(w / drv->VIDEO_MODE_X, h / drv->VIDEO_MODE_Y);
-	Uint8 *data = (Uint8 *)SDL_calloc(1, 32 * mag * mag);
-	Uint8 *mask = (Uint8 *)SDL_calloc(1, 32 * mag * mag);
-	MagBits(data, &MacCursor[4], mag);
-	MagBits(mask, &MacCursor[36], mag);
-	SDL_Cursor *cursor = SDL_CreateCursor(data, mask, 16 * mag, 16 * mag, hot ? MacCursor[2] * mag : 0, hot ? MacCursor[3] * mag : 0);
+	float mag = std::min((float)w / drv->VIDEO_MODE_X, (float)h / drv->VIDEO_MODE_Y);
+	int size = ceilf(16 * mag), n = ((size + 7) >> 3) * size;
+	Uint8 *data = (Uint8 *)SDL_calloc(n, 2);
+	Uint8 *mask = (Uint8 *)SDL_calloc(n, 2);
+	MagBits(data, &MacCursor[4], size);
+	MagBits(mask, &MacCursor[36], size);
+	SDL_Cursor *cursor = SDL_CreateCursor(data, mask, size, size, hot ? MacCursor[2] * mag : 0, hot ? MacCursor[3] * mag : 0);
 	SDL_free(data);
 	SDL_free(mask);
 	return cursor;
 }
 #endif
 
-void driver_base::set_video_mode(int flags)
+void driver_base::set_video_mode(int flags, int pitch)
 {
-	int depth = sdl_depth_of_video_depth(VIDEO_MODE_DEPTH);
-	if ((s = init_sdl_video(VIDEO_MODE_X, VIDEO_MODE_Y, depth, flags)) == NULL)
+	if ((s = init_sdl_video(VIDEO_MODE_X, VIDEO_MODE_Y, VIDEO_MODE_DEPTH, flags, pitch)) == NULL)
 		return;
 #ifdef ENABLE_VOSF
 	the_host_buffer = (uint8 *)s->pixels;
@@ -1023,13 +1045,18 @@ void driver_base::set_video_mode(int flags)
 
 void driver_base::init()
 {
-	set_video_mode(display_type == DISPLAY_SCREEN ? SDL_WINDOW_FULLSCREEN : 0);
+	int pitch = VIDEO_MODE_X;
+	switch (VIDEO_MODE_DEPTH) {
+		case VIDEO_DEPTH_16BIT: pitch <<= 1; break;
+		case VIDEO_DEPTH_32BIT: pitch <<= 2; break;
+	}
+		
 	int aligned_height = (VIDEO_MODE_Y + 15) & ~15;
 
 #ifdef ENABLE_VOSF
 	use_vosf = true;
 	// Allocate memory for frame buffer (SIZE is extended to page-boundary)
-	the_buffer_size = page_extend((aligned_height + 2) * s->pitch);
+	the_buffer_size = page_extend((aligned_height + 2) * pitch);
 	the_buffer = (uint8 *)vm_acquire_framebuffer(the_buffer_size);
 	the_buffer_copy = (uint8 *)malloc(the_buffer_size);
 	D(bug("the_buffer = %p, the_buffer_copy = %p, the_host_buffer = %p\n", the_buffer, the_buffer_copy, the_host_buffer));
@@ -1052,12 +1079,15 @@ void driver_base::init()
 #endif
 	if (!use_vosf) {
 		// Allocate memory for frame buffer
-		the_buffer_size = (aligned_height + 2) * s->pitch;
+		the_buffer_size = (aligned_height + 2) * pitch;
 		the_buffer_copy = (uint8 *)calloc(1, the_buffer_size);
 		the_buffer = (uint8 *)vm_acquire_framebuffer(the_buffer_size);
+		memset(the_buffer, 0, the_buffer_size);
 		D(bug("the_buffer = %p, the_buffer_copy = %p\n", the_buffer, the_buffer_copy));
 	}
 
+	set_video_mode(display_type == DISPLAY_SCREEN ? SDL_WINDOW_FULLSCREEN : 0, pitch);
+
 	// Set frame buffer base
 	set_mac_frame_buffer(monitor, VIDEO_MODE_DEPTH, true);
 
@@ -1067,6 +1097,8 @@ void driver_base::init()
 	sdl_palette = SDL_AllocPalette(256);
 	sdl_palette->colors[1] = (SDL_Color){ .r = 0, .g = 0, .b = 0, .a = 255 };
 	SDL_SetSurfacePalette(s, sdl_palette);
+
+	if (PrefsFindBool("init_grab") && !PrefsFindBool("hardcursor")) grab_mouse();
 }
 
 void driver_base::adapt_to_video_mode() {
@@ -1672,6 +1704,9 @@ static void do_toggle_fullscreen(void)
 			int m = get_mag_rate();
 			SDL_SetWindowSize(sdl_window, m * VIDEO_MODE_X, m * VIDEO_MODE_Y);
 			SDL_SetWindowGrab(sdl_window, SDL_FALSE);
+#ifndef __MACOSX__
+			SDL_SetWindowPosition(sdl_window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
+#endif
 		} else {
 			display_type = DISPLAY_SCREEN;
 			SDL_SetWindowFullscreen(sdl_window, SDL_WINDOW_FULLSCREEN_DESKTOP);
@@ -1995,7 +2030,7 @@ void SDL_monitor_desc::switch_to_current_mode(void)
 #ifdef SHEEPSHAVER
 bool video_can_change_cursor(void)
 {
-	return PrefsFindBool("hardcursor") && (display_type == DISPLAY_WINDOW || PrefsFindBool("scale_integer"));
+	return PrefsFindBool("hardcursor");
 }
 #endif
 
@@ -2412,6 +2447,11 @@ static void handle_events(void)
 				break;
 			}
 
+			case SDL_DROPFILE:
+				CDROMDrop(event.drop.file);
+				SDL_free(event.drop.file);
+				break;
+
 			// Window "close" widget clicked
 			case SDL_QUIT:
 				if (SDL_GetModState() & (KMOD_LALT | KMOD_RALT)) break;
@@ -2587,6 +2627,7 @@ static void update_display_static(driver_base *drv)
 static void update_display_static_bbox(driver_base *drv)
 {
 	const VIDEO_MODE &mode = drv->mode;
+	bool blit = (int)VIDEO_MODE_DEPTH == VIDEO_DEPTH_16BIT;
 
 	// Allocate bounding boxes for SDL_UpdateRects()
 	const uint32 N_PIXELS = 64;
@@ -2619,7 +2660,7 @@ static void update_display_static_bbox(driver_base *drv)
 				const uint32 dst_yb = j * dst_bytes_per_row;
 				if (memcmp(&the_buffer[yb + xb], &the_buffer_copy[yb + xb], xs) != 0) {
 					memcpy(&the_buffer_copy[yb + xb], &the_buffer[yb + xb], xs);
-					Screen_blit((uint8 *)drv->s->pixels + dst_yb + xb, the_buffer + yb + xb, xs);
+					if (blit) Screen_blit((uint8 *)drv->s->pixels + dst_yb + xb, the_buffer + yb + xb, xs);
 					dirty = true;
 				}
 			}
