@@ -540,14 +540,14 @@ static mac_file_handle *open_filehandle(const char *name)
 		return fh;
 }
 
-void *Sys_open(const char *name, bool read_only)
+void *Sys_open(const char *name, bool read_only, bool is_cdrom)
 {
 	bool is_file = strncmp(name, "/dev/", 5) != 0;
 #if defined(__FreeBSD__)
 	                // SCSI                             IDE
-	bool is_cdrom = strncmp(name, "/dev/cd", 7) == 0 || strncmp(name, "/dev/acd", 8) == 0;
+	is_cdrom |= strncmp(name, "/dev/cd", 7) == 0 || strncmp(name, "/dev/acd", 8) == 0;
 #else
-	bool is_cdrom = strncmp(name, "/dev/cd", 7) == 0;
+	is_cdrom |= strncmp(name, "/dev/cd", 7) == 0;
 #endif
 	bool is_floppy = strncmp(name, "/dev/fd", 7) == 0;
 
