@@ -163,47 +163,6 @@ finish_braces (void)
 	close_brace ();
 }
 
-static void 
-pop_braces (int to)
-{
-    while (n_braces > to)
-	close_brace ();
-}
-
-static int 
-bit_size (int size)
-{
-    switch (size)
-    {
-     case sz_byte:
-	return 8;
-     case sz_word:
-	return 16;
-     case sz_long:
-	return 32;
-     default:
-	abort ();
-    }
-    return 0;
-}
-
-static const char *
-bit_mask (int size)
-{
-    switch (size)
-    {
-     case sz_byte:
-	return "0xff";
-     case sz_word:
-	return "0xffff";
-     case sz_long:
-	return "0xffffffff";
-     default:
-	abort ();
-    }
-    return 0;
-}
-
 static __inline__ void gen_update_next_handler(void)
 {
     return; /* Can anything clever be done here? */
@@ -1136,49 +1095,6 @@ genflags (flagtypes type, wordsizes size, char *value, char *src, char *dst)
     }
 }
 
-static void 
-force_range_for_rox (const char *var, wordsizes size)
-{
-    /* Could do a modulo operation here... which one is faster? */
-    switch (size)
-    {
-     case sz_long:
-	comprintf ("\tif (%s >= 33) %s -= 33;\n", var, var);
-	break;
-     case sz_word:
-	comprintf ("\tif (%s >= 34) %s -= 34;\n", var, var);
-	comprintf ("\tif (%s >= 17) %s -= 17;\n", var, var);
-	break;
-     case sz_byte:
-	comprintf ("\tif (%s >= 36) %s -= 36;\n", var, var);
-	comprintf ("\tif (%s >= 18) %s -= 18;\n", var, var);
-	comprintf ("\tif (%s >= 9) %s -= 9;\n", var, var);
-	break;
-    }
-}
-
-static const char *
-cmask (wordsizes size)
-{
-    switch (size)
-    {
-     case sz_byte:
-	return "0x80";
-     case sz_word:
-	return "0x8000";
-     case sz_long:
-	return "0x80000000";
-     default:
-	abort ();
-    }
-}
-
-static int 
-source_is_imm1_8 (struct instr *i)
-{
-    return i->stype == 3;
-}
-
 static int  /* returns zero for success, non-zero for failure */
 gen_opcode (unsigned long int opcode)
 {
@@ -1359,7 +1275,7 @@ gen_opcode (unsigned long int opcode)
      case i_BCLR:
      case i_BSET:
      case i_BTST:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	genamode (curi->smode, "srcreg", curi->size, "src", 1, 0);
 	genamode (curi->dmode, "dstreg", curi->size, "dst", 1, 0);
 	start_brace();
@@ -1541,7 +1457,7 @@ gen_opcode (unsigned long int opcode)
 	failure;
 	break;
      case i_RTD:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	genamode (curi->smode, "srcreg", curi->size, "offs", 1, 0);
 	/* offs is constant */
 	comprintf("\tadd_l_ri(offs,4);\n");
@@ -1557,7 +1473,7 @@ gen_opcode (unsigned long int opcode)
 	isjump; 
 	break;
      case i_LINK:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	genamode (curi->smode, "srcreg", sz_long, "src", 1, 0);
 	genamode (curi->dmode, "dstreg", curi->size, "offs", 1, 0);
 	comprintf("\tsub_l_ri(15,4);\n"
@@ -1569,7 +1485,7 @@ gen_opcode (unsigned long int opcode)
 	genastore ("src", curi->smode, "srcreg", sz_long, "src");
 	break;
      case i_UNLK:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	genamode (curi->smode, "srcreg", curi->size, "src", 1, 0);
 	comprintf("\tmov_l_rr(15,src);\n"
 		  "\treadlong(15,src,scratchie);\n"
@@ -1790,7 +1706,7 @@ gen_opcode (unsigned long int opcode)
 	break;
 
      case i_Scc:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	genamode (curi->smode, "srcreg", curi->size, "src", 2, 0);
 	start_brace ();
 	comprintf ("\tint val = scratchie++;\n");
@@ -1837,7 +1753,7 @@ gen_opcode (unsigned long int opcode)
 	failure;
 	break;
      case i_MULU:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	comprintf("\tdont_care_flags();\n");
 	genamode (curi->smode, "srcreg", sz_word, "src", 1, 0);
 	genamode (curi->dmode, "dstreg", sz_word, "dst", 1, 0);
@@ -1852,7 +1768,7 @@ gen_opcode (unsigned long int opcode)
 	genastore ("dst", curi->dmode, "dstreg", sz_long, "dst");
 	break;
      case i_MULS:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	comprintf("\tdont_care_flags();\n");
 	genamode (curi->smode, "srcreg", sz_word, "src", 1, 0);
 	genamode (curi->dmode, "dstreg", sz_word, "dst", 1, 0);
@@ -1886,7 +1802,7 @@ gen_opcode (unsigned long int opcode)
 	genamode (curi->smode, "srcreg", curi->size, "cnt", 1, 0);
 	genamode (curi->dmode, "dstreg", curi->size, "data", 1, 0);
 	if (curi->smode!=immi) {
-/*		failure; /* UNTESTED: NEW: from "Ipswitch Town" release */
+/*		failure; UNTESTED: NEW: from "Ipswitch Town" release */
 	    if (!noflags) {
 		uses_cmov;
 		start_brace();
@@ -2039,7 +1955,7 @@ gen_opcode (unsigned long int opcode)
 	break;
 
      case i_ASL:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
     mayfail;
 	if (curi->smode==Dreg) {
 	  comprintf("if ((uae_u32)srcreg==(uae_u32)dstreg) {\n"
@@ -2186,7 +2102,7 @@ gen_opcode (unsigned long int opcode)
 	break;
     
      case i_LSR:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
     mayfail;
 	if (curi->smode==Dreg) {
 	  comprintf("if ((uae_u32)srcreg==(uae_u32)dstreg) {\n"
@@ -2337,7 +2253,7 @@ gen_opcode (unsigned long int opcode)
 	genamode (curi->smode, "srcreg", curi->size, "cnt", 1, 0);
 	genamode (curi->dmode, "dstreg", curi->size, "data", 1, 0);
 	if (curi->smode!=immi) {
-/*		failure; /* UNTESTED: NEW: from "Ipswitch Town" release */
+/*		failure; UNTESTED: NEW: from "Ipswitch Town" release */
 	    if (!noflags) {
 		uses_cmov;
 		start_brace();
@@ -2609,7 +2525,7 @@ gen_opcode (unsigned long int opcode)
 	failure;
 	break;
      case i_MULL:
-/*	failure; /* NEW: from "Ipswitch Town" release */
+/*	failure; NEW: from "Ipswitch Town" release */
 	if (!noflags) {
 	    failure;
 	    break;
@@ -3038,6 +2954,9 @@ main (int argc, char **argv)
     stblfile = fopen ("compstbl.cpp", "wb");
     freopen ("compemu.cpp", "wb", stdout);
 
+    fprintf(stblfile, "#if USE_JIT\n");
+    printf("#if USE_JIT\n");
+
     generate_includes (stdout);
     generate_includes (stblfile);
 
@@ -3059,6 +2978,9 @@ main (int argc, char **argv)
     noflags=1;
     generate_func (noflags);
 
+    fprintf(stblfile, "#endif //USE_JIT\n");
+    printf("#endif //USE_JIT\n");
+
     free(opcode_map);
     free(opcode_last_postfix);
     free(opcode_next_clev);
