@@ -252,8 +252,7 @@ static int open_rsrc(const char *path, int flag)
 	make_rsrc_path(path, rsrc_path);
 
 	int fd = open(rsrc_path, flag);
-	if (fd < 0 && flag == O_WRONLY)
-		fd = open(rsrc_path, O_WRONLY | O_CREAT); // for APFS
+	if (fd < 0 && (flag == O_WRONLY || flag == O_RDWR)) fd = open(rsrc_path, flag | O_CREAT); // for APFS
 	return fd;
 }
 
@@ -421,9 +420,9 @@ void get_finfo(const char *path, uint32 finfo, uint32 fxinfo, bool is_dir)
 
 	// No native Finder info, translate file name extension to MacOS type/creator
 	if (!is_dir) {
-		int path_len = strlen(path);
+		size_t path_len = strlen(path);
 		for (int i=0; e2t_translation[i].ext; i++) {
-			int ext_len = strlen(e2t_translation[i].ext);
+			size_t ext_len = strlen(e2t_translation[i].ext);
 			if (path_len < ext_len)
 				continue;
 			if (!strcmp(path + path_len - ext_len, e2t_translation[i].ext)) {
@@ -627,11 +626,11 @@ const char *convert_string(const char *str, CFStringEncoding from, CFStringEncod
 // Convert from the host OS filename encoding to MacRoman
 const char *host_encoding_to_macroman(const char *filename)
 {
-	return convert_string(filename, kCFStringEncodingUTF8, kCFStringEncodingMacRoman);
+	return convert_string(filename, kCFStringEncodingUTF8, PrefsFindInt32("name_encoding"));
 }
 
 // Convert from MacRoman to host OS filename encoding
 const char *macroman_to_host_encoding(const char *filename)
 {
-	return convert_string(filename, kCFStringEncodingMacRoman, kCFStringEncodingUTF8);
+	return convert_string(filename, PrefsFindInt32("name_encoding"), kCFStringEncodingUTF8);
 }
